@@ -25,8 +25,6 @@
 #include "qwt_symbol.h"
 #include "qwt_plot_curve.h"
 
-#define SCALE_PEN 0
-
 #if QT_VERSION < 0x040000
 #include <qguardedptr.h>
 #else
@@ -143,36 +141,6 @@ private:
 
 QMap<QwtPlotCanvas *, QPainter *> QwtGuardedPainter::_map;
 
-static QwtPolygon clipPolygon(QPainter *painter, int attributes, 
-    const QwtPolygon &polygon)
-{
-    bool doClipping = attributes & QwtPlotCurve::ClipPolygons;
-    QRect clipRect = painter->window();
-
-    if ( !doClipping )
-    {
-#if QT_VERSION >= 0x040000
-        const QPaintEngine *pe = painter->paintEngine();
-        if ( pe && pe->type() == QPaintEngine::SVG )
-#else
-        if ( painter->device()->devType() == QInternal::Picture )
-#endif
-        {
-            // The SVG paint engine ignores any clipping,
-            // so we enable polygon clipping.
-
-            doClipping = true;
-            if ( painter->hasClipping() )
-                clipRect &= painter->clipRegion().boundingRect();
-        }
-    }
-
-    if ( doClipping )
-        return QwtClipper::clipPolygon(clipRect, polygon);
-    
-    return polygon;
-}
-
 static int verifyRange(int size, int &i1, int &i2)
 {
     if (size < 1) 
@@ -251,6 +219,8 @@ public:
     int paintAttributes;
 
     QwtGuardedPainter guardedPainter;
+
+    QRect canvasRect; // temporary, while painting
 };
 
 //! Constructor
@@ -262,7 +232,7 @@ QwtPlotCurve::QwtPlotCurve():
 
 /*!
   Constructor
-  \param title title of the curve   
+  \param title Title of the curve   
 */
 QwtPlotCurve::QwtPlotCurve(const QwtText &title):
     QwtPlotItem(title)
@@ -272,7 +242,7 @@ QwtPlotCurve::QwtPlotCurve(const QwtText &title):
 
 /*!
   Constructor
-  \param title title of the curve   
+  \param title Title of the curve   
 */
 QwtPlotCurve::QwtPlotCurve(const QString &title):
     QwtPlotItem(QwtText(title))
@@ -308,27 +278,11 @@ int QwtPlotCurve::rtti() const
 }
 
 /*!
-  \brief Specify an attribute how to draw the curve
-
-  The attributes can be used to modify the drawing algorithm.
-
-  The following attributes are defined:<dl>
-  <dt>PaintFiltered</dt>
-  <dd>Tries to reduce the data that has to be painted, by sorting out
-      duplicates, or paintings outside the visible area. Might have a
-      notable impact on curves with many close points. 
-      Only a couple of very basic filtering algos are implemented.</dd>
-  <dt>ClipPolygons</dt>
-  <dd>Clip polygons before painting them. In situations, where points
-      are far outside the visible area this might be a great improvement
-      for the painting performance ( especially on Windows ).
-  </dl>
-
-  The default is, that no paint attributes are enabled.
+  Specify an attribute how to draw the curve
 
   \param attribute Paint attribute
   \param on On/Off
-  /sa testPaintAttribute()
+  /sa PaintAttribute, testPaintAttribute()
 */
 void QwtPlotCurve::setPaintAttribute(PaintAttribute attribute, bool on)
 {
@@ -340,7 +294,7 @@ void QwtPlotCurve::setPaintAttribute(PaintAttribute attribute, bool on)
 
 /*!
     \brief Return the current paint attributes
-    \sa setPaintAttribute
+    \sa PaintAttribute, setPaintAttribute()
 */
 bool QwtPlotCurve::testPaintAttribute(PaintAttribute attribute) const
 {
@@ -348,31 +302,10 @@ bool QwtPlotCurve::testPaintAttribute(PaintAttribute attribute) const
 }
 
 /*!
-  \brief Set the curve's drawing style
+  Set the curve's drawing style
 
-  Valid styles are:
-  <dl>
-  <dt>NoCurve</dt>
-  <dd>Don't draw a curve. Note: This doesn't affect the symbol. </dd>
-  <dt>Lines</dt>
-  <dd>Connect the points with straight lines. The lines might
-      be interpolated depending on the 'Fitted' option. Curve
-      fitting can be configured using setCurveFitter.</dd>
-  <dt>Sticks</dt>
-  <dd>Draw vertical sticks from a baseline which is defined by setBaseline().</dd>
-  <dt>Steps</dt>
-  <dd>Connect the points with a step function. The step function
-      is drawn from the left to the right or vice versa,
-      depending on the 'Inverted' option.</dd>
-  <dt>Dots</dt>
-  <dd>Draw dots at the locations of the data points. Note:
-      This is different from a dotted line (see setPen()).</dd>
-  <dt>UserCurve ...</dt>
-  <dd>Styles >= UserCurve are reserved for derived
-      classes of QwtPlotCurve that overload drawCurve() with
-      additional application specific curve types.</dd>
-  </dl>
-  \sa style()
+  \param style Curve style
+  \sa CurveStyle, style()
 */
 void QwtPlotCurve::setStyle(CurveStyle style)
 {
@@ -384,8 +317,8 @@ void QwtPlotCurve::setStyle(CurveStyle style)
 }
 
 /*!
-    \brief Return the current style
-    \sa setStyle()
+    Return the current style
+    \sa CurveStyle, setStyle()
 */
 QwtPlotCurve::CurveStyle QwtPlotCurve::style() const 
 { 
@@ -414,9 +347,13 @@ const QwtSymbol &QwtPlotCurve::symbol() const
 }
 
 /*!
-  \brief Assign a pen
+  Assign a pen
+
+  The width of non cosmetic pens is scaled according to the resolution
+  of the paint device.
+
   \param pen New pen
-  \sa pen(), brush()
+  \sa pen(), brush(), QwtPainter::scaledPen()
 */
 void QwtPlotCurve::setPen(const QPen &pen)
 {
@@ -438,13 +375,16 @@ const QPen& QwtPlotCurve::pen() const
 
 /*!
   \brief Assign a brush. 
-         In case of brush.style() != QBrush::NoBrush 
-         and style() != QwtPlotCurve::Sticks
-         the area between the curve and the baseline will be filled.
-         In case !brush.color().isValid() the area will be filled by
-         pen.color(). The fill algorithm simply connects the first and the
-         last curve point to the baseline. So the curve data has to be sorted 
-         (ascending or descending). 
+
+   In case of brush.style() != QBrush::NoBrush 
+   and style() != QwtPlotCurve::Sticks
+   the area between the curve and the baseline will be filled.
+
+   In case !brush.color().isValid() the area will be filled by
+   pen.color(). The fill algorithm simply connects the first and the
+   last curve point to the baseline. So the curve data has to be sorted 
+   (ascending or descending). 
+
   \param brush New brush
   \sa brush(), setBaseline(), baseline()
 */
@@ -470,13 +410,13 @@ const QBrush& QwtPlotCurve::brush() const
 /*!
   Set data by copying x- and y-values from specified memory blocks.
   Contrary to setCurveRawData(), this function makes a 'deep copy' of
-  the data.
+  the data. 
 
-  \param xData pointer to x values
-  \param yData pointer to y values
-  \param size size of xData and yData
+  \param xData Pointer to x values
+  \param yData Pointer to y values
+  \param size Size of xData and yData
 
-  \sa QwtCPointerData
+  \note Internally the data is stored in a QwtArrayData object
 */
 void QwtPlotCurve::setData(const double *xData, const double *yData, int size)
 {
@@ -486,12 +426,13 @@ void QwtPlotCurve::setData(const double *xData, const double *yData, int size)
 }
 
 /*!
-  \brief Initialize data with x- and y-arrays (explicitly shared)
+  Initialize data with x- and y-arrays (explicitly shared)
+  ( Builds an QwtArrayData object internally )
 
   \param xData x data
   \param yData y data
 
-  \sa QwtArrayData
+  \note Internally the data is stored in a QwtArrayData object
 */
 void QwtPlotCurve::setData(const QwtArray<double> &xData, 
     const QwtArray<double> &yData)
@@ -505,7 +446,7 @@ void QwtPlotCurve::setData(const QwtArray<double> &xData,
   Initialize data with an array of points (explicitly shared).
 
   \param data Data
-  \sa QwtPolygonFData
+  \note Internally the data is stored in a QwtPolygonFData object
 */
 #if QT_VERSION < 0x040000
 void QwtPlotCurve::setData(const QwtArray<QwtDoublePoint> &data)
@@ -542,7 +483,7 @@ void QwtPlotCurve::setData(const QwtData &data)
   \param yData pointer to y data
   \param size size of x and y
 
-  \sa QwtCPointerData::setData.
+  \note Internally the data is stored in a QwtCPointerData object
 */
 void QwtPlotCurve::setRawData(const double *xData, const double *yData, int size)
 {
@@ -576,9 +517,11 @@ QwtDoubleRect QwtPlotCurve::boundingRect() const
 */
 void QwtPlotCurve::draw(QPainter *painter,
     const QwtScaleMap &xMap, const QwtScaleMap &yMap,
-    const QRect &) const
+    const QRect &canvasRect) const
 {
+    d_data->canvasRect = canvasRect;
     draw(painter, xMap, yMap, 0, -1);
+    d_data->canvasRect = QRect();
 }
 
 /*!
@@ -697,18 +640,7 @@ void QwtPlotCurve::draw(QPainter *painter,
     if ( verifyRange(dataSize(), from, to) > 0 )
     {
         painter->save();
-        
-        QPen pen = d_data->pen;
-
-#if SCALE_PEN
-        if ( pen.width() > 0 )
-        {
-            const QwtMetricsMap &metricsMap = QwtPainter::metricsMap();
-            pen.setWidth(metricsMap.screenToLayoutX(pen.width()));
-        }
-#endif
-            
-        painter->setPen(pen);
+        painter->setPen(QwtPainter::scaledPen(d_data->pen));
 
         /*
           Qt 4.0.0 is slow when drawing lines, but it's even 
@@ -891,7 +823,8 @@ void QwtPlotCurve::drawLines(QPainter *painter,
         }
     }
 
-    polyline = ::clipPolygon(painter, d_data->paintAttributes, polyline);
+    if ( d_data->canvasRect.isValid() && d_data->paintAttributes & ClipPolygons )
+        polyline = QwtClipper::clipPolygon(d_data->canvasRect, polyline);
 
     QwtPainter::drawPolyline(painter, polyline);
 
@@ -944,10 +877,6 @@ void QwtPlotCurve::drawDots(QPainter *painter,
     const QwtScaleMap &xMap, const QwtScaleMap &yMap, 
     int from, int to) const
 {
-    const QRect window = painter->window();
-    if ( window.isEmpty() )
-        return;
-
     const bool doFill = d_data->brush.style() != Qt::NoBrush;
 
     QwtPolygon polyline;
@@ -956,7 +885,7 @@ void QwtPlotCurve::drawDots(QPainter *painter,
 
     if ( to > from && d_data->paintAttributes & PaintFiltered )
     {
-        if ( doFill )   
+        if ( doFill || d_data->canvasRect.isEmpty() )   
         {
             QPoint pp( xMap.transform(x(from)), yMap.transform(y(from)) );
 
@@ -985,7 +914,7 @@ void QwtPlotCurve::drawDots(QPainter *painter,
             // if we don't need to fill, we can sort out
             // duplicates independent from the order
 
-            PrivateData::PixelMatrix pixelMatrix(window);
+            PrivateData::PixelMatrix pixelMatrix(d_data->canvasRect);
 
             for (int i = from; i <= to; i++)
             {
@@ -1012,7 +941,9 @@ void QwtPlotCurve::drawDots(QPainter *painter,
 
     if ( doFill )
     {
-        polyline = ::clipPolygon(painter, d_data->paintAttributes, polyline);
+        if ( d_data->canvasRect.isValid() && (d_data->paintAttributes & ClipPolygons) )
+            polyline = QwtClipper::clipPolygon(d_data->canvasRect, polyline);
+
         fillCurve(painter, xMap, yMap, polyline);
     }
 }
@@ -1058,7 +989,8 @@ void QwtPlotCurve::drawSteps(QPainter *painter,
         polyline.setPoint(ip, xi, yi);
     }
 
-    polyline = ::clipPolygon(painter, d_data->paintAttributes, polyline);
+    if ( d_data->canvasRect.isValid() && (d_data->paintAttributes & ClipPolygons) )
+        polyline = QwtClipper::clipPolygon(d_data->canvasRect, polyline);
 
     QwtPainter::drawPolyline(painter, polyline);
 
@@ -1068,26 +1000,12 @@ void QwtPlotCurve::drawSteps(QPainter *painter,
 
 
 /*!
-  \brief Specify an attribute for drawing the curve
-
-  The attributes can be used to modify the drawing style.
-  The following attributes are defined:<dl>
-  <dt>Fitted</dt>
-  <dd>For Lines only. A QwtCurveFitter tries to
-      interpolate/smooth the curve, before it is painted.
-      Note that curve fitting requires temorary memory
-      for calculating coefficients and additional points. 
-      If painting in Fitted mode is slow it might be better
-      to fit the points, before they are passed to QwtPlotCurve.
-  </dd>
-  <dt>Inverted</dt>
-  <dd>For Steps only. Draws a step function
-      from the right to the left.</dd></dl>
+  Specify an attribute for drawing the curve
 
   \param attribute Curve attribute
   \param on On/Off
 
-  /sa testCurveAttribute(), setCurveFitter()
+  /sa CurveAttribute, testCurveAttribute(), setCurveFitter()
 */
 void QwtPlotCurve::setCurveAttribute(CurveAttribute attribute, bool on)
 {
@@ -1104,7 +1022,7 @@ void QwtPlotCurve::setCurveAttribute(CurveAttribute attribute, bool on)
 
 /*!
     \return true, if attribute is enabled
-    \sa setCurveAttribute()
+    \sa CurveAttribute, setCurveAttribute()
 */
 bool QwtPlotCurve::testCurveAttribute(CurveAttribute attribute) const 
 { 
@@ -1114,18 +1032,8 @@ bool QwtPlotCurve::testCurveAttribute(CurveAttribute attribute) const
 /*!
   Assign the curve type
 
-  <dt>QwtPlotCurve::Yfx
-  <dd>Draws y as a function of x (the default). The
-      baseline is interpreted as a horizontal line
-      with y = baseline().</dd>
-  <dt>QwtPlotCurve::Xfy
-  <dd>Draws x as a function of y. The baseline is
-      interpreted as a vertical line with x = baseline().</dd>
-
-  The baseline is used for aligning the sticks, or
-  filling the curve with a brush.
-
-  \sa curveType()
+  \param curveType Yfx or Xfy
+  \sa CurveType, curveType()
 */
 void QwtPlotCurve::setCurveType(CurveType curveType)
 {
@@ -1138,13 +1046,19 @@ void QwtPlotCurve::setCurveType(CurveType curveType)
 
 /*!
    Return the curve type
-   \sa setCurveType()
+   \sa CurveType, setCurveType()
 */
 QwtPlotCurve::CurveType QwtPlotCurve::curveType() const
 {
     return d_data->curveType;
 }
 
+/*!
+  Assign a curve fitter
+  setCurveFitter(NULL) disables curve fitting.
+
+  \param curveFitter Curve fitter
+*/
 void QwtPlotCurve::setCurveFitter(QwtCurveFitter *curveFitter)
 {
     delete d_data->curveFitter;
@@ -1153,6 +1067,10 @@ void QwtPlotCurve::setCurveFitter(QwtCurveFitter *curveFitter)
     itemChanged();
 }
 
+/*!
+  Get the curve fitter. If curve fitting is disabled NULL is returned.
+  \return Curve fitter
+*/
 QwtCurveFitter *QwtPlotCurve::curveFitter() const
 {
     return d_data->curveFitter;
@@ -1169,7 +1087,6 @@ QwtCurveFitter *QwtPlotCurve::curveFitter() const
 
   \sa setBrush(), setBaseline(), setCurveType()
 */
-
 void QwtPlotCurve::fillCurve(QPainter *painter,
     const QwtScaleMap &xMap, const QwtScaleMap &yMap,
     QwtPolygon &pa) const
@@ -1203,7 +1120,6 @@ void QwtPlotCurve::fillCurve(QPainter *painter,
   \param yMap Y map
   \param pa Polygon to be completed
 */
-
 void QwtPlotCurve::closePolyline(
     const QwtScaleMap &xMap, const QwtScaleMap &yMap,
     QwtPolygon &pa) const
@@ -1246,28 +1162,17 @@ void QwtPlotCurve::drawSymbols(QPainter *painter, const QwtSymbol &symbol,
     int from, int to) const
 {
     painter->setBrush(symbol.brush());
+    painter->setPen(QwtPainter::scaledPen(symbol.pen()));
 
     const QwtMetricsMap &metricsMap = QwtPainter::metricsMap();
 
-    QPen pen = symbol.pen();
-
-#if SCALE_PEN
-    if ( pen.width() > 0 )
-        pen.setWidth(metricsMap.screenToLayoutX(pen.width()));
-#endif
-
-    painter->setPen(pen);
-
     QRect rect;
     rect.setSize(metricsMap.screenToLayout(symbol.size()));
 
-    if ( to > from && d_data->paintAttributes & PaintFiltered )
+    if ( (to > from) && (d_data->paintAttributes & PaintFiltered) &&
+         d_data->canvasRect.isValid() )
     {
-        const QRect window = painter->window();
-        if ( window.isEmpty() )
-            return;
-
-        PrivateData::PixelMatrix pixelMatrix(window);
+        PrivateData::PixelMatrix pixelMatrix(d_data->canvasRect);
 
         for (int i = from; i <= to; i++)
         {
@@ -1318,7 +1223,7 @@ void QwtPlotCurve::setBaseline(double reference)
 
 /*!
     Return the value of the baseline
-    \sa setBaseline
+    \sa setBaseline()
 */
 double QwtPlotCurve::baseline() const 
 { 
@@ -1334,6 +1239,17 @@ int QwtPlotCurve::dataSize() const
     return d_xy->size();
 }
 
+/*!
+  Find the closest curve point for a specific position
+
+  \param pos Position, where to look for the closest curve point
+  \param dist If dist != NULL, closestPoint() returns the distance between
+              the position and the clostest curve point
+  \return Index of the closest curve point, or -1 if none can be found 
+          ( f.e when the curve has no points )
+  \note closestPoint() implements a dumb algorithm, that iterates 
+        over all points
+*/
 int QwtPlotCurve::closestPoint(const QPoint &pos, double *dist) const
 {
     if ( plot() == NULL || dataSize() <= 0 )
